"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
/** @typedef {(entries: Object[], observer: PerformanceObserver) => any} EntriesCallback */
/** @typedef {(entry: PerformanceEntry) => any} EntryCallback */
/** @typedef {'element' | 'first-input' | 'largest-contentful-paint' | 'layout-shift' | 'longtask' | 'mark' | 'measure' | 'navigation' | 'paint' | 'resource'} EntryType */
/** @typedef {{ type: EntryType, buffered?: boolean }} EntryOpts */
const PO = typeof PerformanceObserver === 'undefined' ? null : PerformanceObserver;
const legacySupportedEntryTypes = ['mark', 'measure', 'resource', 'navigation'];
const supportedEntryTypes = legacySupportedEntryTypes.concat([
    'element',
    'first-input',
    'largest-contentful-paint',
    'layout-shift',
    'longtask',
    'paint'
]);
/**
 * Create performance observer.
 *
 * @param {EntryOpts} opts
 * @param {EntriesCallback} callback
 * @return {PerformanceObserver}
 */
function observeEntries(opts, callback) {
    const type = normalizeEntryType(opts.type);
    if (!PO)
        return createFakeObserver();
    try {
        const isLegacyType = legacySupportedEntryTypes.indexOf(type) !== -1;
        const supportedTypes = PO.supportedEntryTypes || legacySupportedEntryTypes;
        if (!isLegacyType && supportedTypes.indexOf(type) === -1)
            return createFakeObserver();
        /** @type {PerformanceObserver} */
        const observer = new PO(list => callback(list.getEntries(), observer));
        const observerOpts = isLegacyType ? { entryTypes: [type] } : opts;
        utils_1.debug('new PO(%o)', observerOpts);
        observer.observe(observerOpts);
        if (opts.buffered && isLegacyType) {
            utils_1.raf(() => {
                if (!utils_1.perf)
                    return;
                utils_1.debug('emit buffered');
                callback(utils_1.perf.getEntriesByType(type), observer);
            });
        }
        return observer;
    }
    catch (err) {
        utils_1.warn(err);
        return createFakeObserver();
    }
}
exports.observeEntries = observeEntries;
/**
 * Get buffered entries by `type`.
 *
 * @param {EntryType} entryType
 * @return {Promise<Object[]>}
 */
function getEntriesByType(entryType) {
    const type = normalizeEntryType(entryType);
    return new Promise(resolve => {
        if (utils_1.perf && legacySupportedEntryTypes.indexOf(type) >= 0) {
            utils_1.debug('use sync API');
            return resolve(utils_1.perf.getEntriesByType(type));
        }
        if (type === 'longtask' || !PO)
            return resolve([]); // no buffering for longTasks
        const observer = observeEntries({ type, buffered: true }, events => {
            observer.disconnect();
            clearTimeout(timeout);
            resolve(events);
        });
        const timeout = setTimeout(() => {
            observer.disconnect();
            utils_1.debug('got events timeout');
            resolve([]);
        }, 250);
    });
}
exports.getEntriesByType = getEntriesByType;
/**
 * Create a fake performance observer interface when the object is not available.
 * The behaviour is similar to Firefox when `type` is not supported.
 *
 * @return {PerformanceObserver}
 */
function createFakeObserver() {
    return {
        observe() { },
        disconnect() { },
        takeRecords() {
            return [];
        }
    };
}
/**
 * Resolve event type to supported event strings.
 *
 * @param {EntryType} entryType
 * @return {EntryType}
 */
function normalizeEntryType(entryType) {
    const type = entryType.toLowerCase();
    if (supportedEntryTypes.indexOf(type) === -1)
        throw new Error(`Invalid event: ${entryType}`);
    // @ts-ignore
    return type;
}
