/** @typedef {{ entryType: "mark", name: string, startTime: number }} UserTimingMark */
/** @typedef {{ entryType: "measure", name: string, startTime: number, duration: number }} UserTimingMeasure */
/** @typedef {(measure: UserTimingMeasure | null) => any} UserTimingTimeEndPaintCallback */
/**
 * Create a custom performance mark with `markName` name.
 * https://developer.mozilla.org/en-US/docs/Web/API/Performance/mark
 *
 * @param {string} markName
 * @return {UserTimingMark | null}
 */
export function mark(markName: string): {
    entryType: "mark";
    name: string;
    startTime: number;
} | null;
/**
 * Create performance measurement `measureName` between marks, the navigation start time, or `startMarkName`.
 * https://developer.mozilla.org/en-US/docs/Web/API/Performance/measure
 *
 * @param {string} measureName
 * @param {string} [startMarkName]
 * @param {string} [endMarkName]
 * @return {UserTimingMeasure | null}
 */
export function measure(measureName: string, startMarkName?: string | undefined, endMarkName?: string | undefined): {
    entryType: "measure";
    name: string;
    startTime: number;
    duration: number;
} | null;
/**
 * Get the time elapsed since the session start.
 *
 * https://developer.mozilla.org/en-US/docs/Web/API/Performance/now
 */
export function now(): number;
/**
 * Start time measurement.
 * It's similar to console.time(label).
 *
 * @param {string} label
 * @return {UserTimingMark | null}
 */
export function time(label: string): {
    entryType: "mark";
    name: string;
    startTime: number;
} | null;
/**
 * End time measurement.
 * It's similar to console.timeEnd(label).
 *
 * @param {string} label
 * @return {UserTimingMeasure | null}
 */
export function timeEnd(label: string): {
    entryType: "measure";
    name: string;
    startTime: number;
    duration: number;
} | null;
/**
 * End time measurement after the last paint.
 * It's similar to console.timeEnd(label) but async.
 *
 * @param {string} label
 * @param {UserTimingTimeEndPaintCallback} [callback]
 */
export function timeEndPaint(label: string, callback?: ((measure: {
    entryType: "measure";
    name: string;
    startTime: number;
    duration: number;
} | null) => any) | undefined): void;
export type UserTimingMark = {
    entryType: "mark";
    name: string;
    startTime: number;
};
export type UserTimingMeasure = {
    entryType: "measure";
    name: string;
    startTime: number;
    duration: number;
};
export type UserTimingTimeEndPaintCallback = (measure: {
    entryType: "measure";
    name: string;
    startTime: number;
    duration: number;
} | null) => any;
